import { __read } from "tslib";
import { Observable, of, ReplaySubject, Subscription, } from 'rxjs';
import { distinctUntilChanged, filter, map, shareReplay, switchMap, tap, withLatestFrom, } from 'rxjs/operators';
/**
 * RenderAware
 *
 * @description
 * This function returns an object that holds all the shared logic for the push pipe and the let directive
 * responsible for change detection
 * If you extend this class you need to implement how the update of the rendered value happens.
 * Also custom behaviour is something you need to implement in the extending class
 */
export function createRenderAware(cfg) {
    var strategyName$ = new ReplaySubject(1);
    var currentStrategy;
    var strategy$ = strategyName$.pipe(distinctUntilChanged(), switchMap(function (stringOrObservable) {
        return typeof stringOrObservable === 'string'
            ? of(stringOrObservable)
            : stringOrObservable;
    }), map(function (strategy) {
        var s = cfg.strategies[strategy];
        if (!!s) {
            return s;
        }
        throw new Error("Strategy " + strategy + " does not exist.");
    }), tap(function (s) { return (currentStrategy = s); }), 
    // do not repeat the steps before for each subscriber
    shareReplay({ bufferSize: 1, refCount: true }));
    var observablesFromTemplate$ = new ReplaySubject(1);
    var valuesFromTemplate$ = observablesFromTemplate$.pipe(distinctUntilChanged());
    var firstTemplateObservableChange = true;
    var renderingEffect$ = valuesFromTemplate$.pipe(
    // handle null | undefined assignment and new Observable reset
    map(function (observable$) {
        if (observable$ === null) {
            return of(null);
        }
        if (!firstTemplateObservableChange) {
            cfg.resetObserver.next();
            if (observable$ === undefined) {
                return of(undefined);
            }
        }
        firstTemplateObservableChange = false;
        return observable$;
    }), 
    // forward only observable values
    filter(function (o$) { return o$ !== undefined; }), distinctUntilChanged(), switchMap(function (o$) {
        return o$
            // Added behavior will get applied to the observable in `renderWithLatestStrategy`
            .pipe(
        // Forward only distinct values
        distinctUntilChanged(), 
        // Update completion, error and next
        tap(cfg.updateObserver), renderWithLatestStrategy(strategy$));
    }));
    return {
        nextPotentialObservable: function (value) {
            observablesFromTemplate$.next(value);
        },
        nextStrategy: function (nextConfig) {
            strategyName$.next(nextConfig);
        },
        activeStrategy$: strategy$,
        subscribe: function () {
            return new Subscription().add(renderingEffect$.subscribe());
        },
    };
}
function renderWithLatestStrategy(strategyChanges$) {
    return function (o$) {
        return o$.pipe(handleErrorAndComplete(), withLatestFrom(strategyChanges$), 
        // always use latest strategy on value change
        switchMap(function (_a) {
            var _b = __read(_a, 2), renderValue = _b[0], strategy = _b[1];
            return of(renderValue).pipe(strategy.rxScheduleCD);
        }));
    };
    function handleErrorAndComplete() {
        return function (o$) {
            return new Observable(function (subscriber) {
                var subscription = o$.subscribe({
                    next: function (val) { return subscriber.next(val); },
                    // make "error" and "complete" notifications comply with `rxScheduleCD`
                    error: function (err) {
                        console.error(err);
                        subscriber.next();
                    },
                    complete: function () { return subscriber.next(); },
                });
                return function () { return subscription.unsubscribe(); };
            });
        };
    }
}
//# sourceMappingURL=render-aware_creator.js.map